extern crate proc_macro;

use proc_macro::TokenStream;
use quote::quote;

/// Compile time validation of PromQL queries
///
/// The macro takes a query as input (unquoted), and returns a `ValidatedQuery<&'static str>` if it's valid
/// or generates a compiler error based on what's wrong with the provided query.
///
/// Note, in its current implementation, the macro is recommended only for development purposes.
///
/// # Example
/// ```
/// let query = promql!(sum(http_requests)); // returns `ValidatedQuery<&str>`
/// ```
#[proc_macro]
pub fn promql(stream: TokenStream) -> TokenStream {
    let query = stream.to_string();
    match promql_parser::parser::parse(&query) {
        Ok(_) => quote!(::mquery::ValidatedQuery(#query)).into(),
        Err(e) => quote!(compile_error!(#e)).into(),
    }
}
